#!/bin/bash
. /var/packages/MailPlus-Server/target/backend_hook/hookUtils.conf
loadEnv

## Check identity
NODE_ID=$(getHostID)
mailer_servers="$(${MAILPLUS_SERVER_BACKEND_BINARY} --getConfKeyVal "mailer")"
if ! containString "${NODE_ID}" "${mailer_servers}"; then
	exit 0
fi

## BitDefender pseudo key
is_bitdefender_changed=0
if isKeyChanged "spam_engine"; then
	is_bitdefender_changed=1
fi

## Convert syno config
if ! ${SET_CONF_BINARY} "smtp"; then
	err_log "Failed to set conf smtp"
	exit 1
fi

if ! ${SET_CONF_BINARY} "dovecot"; then
	err_log "Failed to set conf dovecot"
	exit 1
fi

if ! ${SET_CONF_BINARY} "rspamd"; then
	err_log "Failed to set conf rspamd"
	exit 1
fi

if isKeyChanged "sync_spam_db"; then
	chown -R postfix:root /var/spool/@MailPlus-Server/MailScanner/spamassassin/*
fi


## Actions

if [[ "${is_bitdefender_changed}" -eq 1 ]]; then
	# For greylist
	if ! ${CALLBACK_BINARY} "smtp" "${_MAIL_mailserver_trigger_events}"; then
		err_log "Failed to callback smtp"
		exit 1
	fi
fi

# MailPlus Server #3338
# syno_action rspamd need to be executed before syno_action antivirus
# to ensure ClamAV start before ClamAV update.
if ! ${CALLBACK_BINARY} "rspamd" "${_MAIL_mailserver_trigger_events}"; then
	err_log "Failed to callback rspamd"
	exit 1
fi

if isKeyChanged "anti_virus_enable" || isKeyChanged "anti_virus_engine"; then
	if ! ${CALLBACK_BINARY} "antivirus" "${_MAIL_mailserver_trigger_events}"; then
		err_log "Failed to callback antivirus"
		exit 1
	fi
fi

if isKeyChanged "report_learn_clear_history"; then
	/var/packages/MailPlus-Server/target/scripts/ReportLearn.py clear_history
fi

if isKeyChanged "spam_enable" || isKeyChanged "spam_auto_learn" || isKeyChanged "spam_report_enable" || [[ "${is_bitdefender_changed}" -eq 1 ]]; then
	if ! ${CALLBACK_BINARY} "setReportLearnMaintainTask" "${_MAIL_mailserver_trigger_events}"; then
		err_log "Failed to setReportLearnMaintainTask"
	fi
fi

if isKeyChanged "spam_enable" || isKeyChanged "spam_auto_learn" || isKeyChanged "spam_report_enable" || isKeyChanged "report_autolearn_enable" || isKeyChanged "report_autolearn_hour" || isKeyChanged "report_autolearn_minute" || [[ "${is_bitdefender_changed}" -eq 1 ]]; then
	if ! ${CALLBACK_BINARY} "setReportAutoLearnTask" "${_MAIL_mailserver_trigger_events}"; then
		err_log "Failed to setReportAutoLearnTask"
	fi
fi

if isKeyChanged "enable_dangerous_content_scan" || isKeyChanged "find_phishing_fraud" || [[ "${is_bitdefender_changed}" -eq 1 ]]; then
	if ! ${CALLBACK_BINARY} "setPhishingUpdateTask" "${_MAIL_mailserver_trigger_events}"; then
		err_log "Failed to setPhishingUpdateTask"
	fi
fi

exit 0
