#!/bin/bash
. /var/packages/MailPlus-Server/target/backend_hook/hookUtils.conf
loadEnv

USERPOLICY_CMD="${PRIVATE_LOCATION}/bin/syno_personal_policy genPoliciesConf"

# redis key
ATTACHMENT_LIMIT="attachment_size_list-"
DISABLE_AUTO_FORWARD="smtp_auto_forward_disable_list-"
SEND_FLOW_LIMIT="daily_flow_limit_list-"
SEND_LOCAL_ONLY="smtp_local_enable_list-"
SEND_QUOTA_LIMIT="daily_quota_list-"

DISABLE_IMAP_LOGIN="imap_enable_list-"
DISABLE_POP3_LOGIN="pop3_enable_list-"
IMAP_LOGIN_LOCAL_ONLY="imap_local_enable_list-"
MAILBOX_QUOTA_LIMIT="mailbox_quota_list-"
POP3_LOGIN_LOCAL_ONLY="pop3_local_enable_list-"

LOCAL_NET="local_net"
LOCAL_PERMIT_DOMAIN="local_permit_domain"

IP_LIST="node_ip_list-"
BIND_IF="node_bind_if-"

MULTIPLE_DOMAIN="virtual_multiple_domain"
ADDITIONAL_DOMAIN="additional_domain-"

regen_dovecot_flags=()
regen_postfix_flags=()

setConfGenFlag()
{

	local dovecot_policies=(
		${DISABLE_IMAP_LOGIN}
		${DISABLE_POP3_LOGIN}
		${IMAP_LOGIN_LOCAL_ONLY}
		${POP3_LOGIN_LOCAL_ONLY}
	)

	local postfix_policies=(
		${ATTACHMENT_LIMIT}
		${DISABLE_AUTO_FORWARD}
		${SEND_FLOW_LIMIT}
		${SEND_LOCAL_ONLY}
		${SEND_QUOTA_LIMIT}
	)

	local policy

	for policy in "${dovecot_policies[@]}"
	do
		if isPrefixKeyChanged "${policy}"; then
			regen_dovecot_flags+=" ${policy}"
		fi
	done
	for policy in "${postfix_policies[@]}"
	do
		if isPrefixKeyChanged "${policy}"; then
			regen_postfix_flags+=" ${policy}"
		fi
	done
}

setDomainRelativeConfFlag()
{
	if isKeyChanged "${MULTIPLE_DOMAIN}" || isPrefixKeyChanged "${ADDITIONAL_DOMAIN}" ; then
		local send_local="$(${MAILPLUS_SERVER_BACKEND_BINARY} --getConfKeyVal "smtpd_send_local_only")"
		if [ "${send_local}" = "yes" ]; then
			regen_postfix_flags+=" ${LOCAL_PERMIT_DOMAIN}"
		fi
	fi
}

setIpRelativeConfFlag()
{
	if isPrefixKeyChanged "${IP_LIST}" || isPrefixKeyChanged "${BIND_IF}" ; then
		local auth_local="$(${MAILPLUS_SERVER_BACKEND_BINARY} --getConfKeyVal "enable_dovecot_auth_local")"
		if [ "${auth_local}" = "yes" ]; then
			regen_dovecot_flags+=" ${LOCAL_NET}"
		fi
	fi
}

if isTreeChanged "user_policy_setting"; then
	setConfGenFlag
fi

if isTreeChanged "virtual_domain"; then
	setDomainRelativeConfFlag
fi

if isTreeChanged "general_config"; then
	setIpRelativeConfFlag
fi

regen_flags=(
	${regen_dovecot_flags}
	${regen_postfix_flags}
)

if [ ${#regen_flags[@]} -ne 0 ] && ! echo "${regen_flags[*]}" | ${USERPOLICY_CMD} > /dev/null; then
	err_log "failed to generate user policy conf"
	exit 1
fi

ret=0
if [ ${#regen_dovecot_flags[@]} -ne 0 ] && ! ${CALLBACK_BINARY} "dovecot_reload" "${trigger_keys}"; then
	err_log "failed to callback dovecot reload"
	ret=1
fi
if [ ${#regen_postfix_flags[@]} -ne 0 ] && ! ${CALLBACK_BINARY} "postfix_reload" "${trigger_keys}"; then
	err_log "failed to callback postfix reload"
	ret=1
fi
exit ${ret}
