#!/bin/bash

###############################################################
#	This script should be hooked when invoking SYNOUserSet()
#	Usable environment variable:
#		NITEMS, ORIGIN_USER_NAME_{n}, USER_NAME_{n}, UID_{n},
#		USER_OP_RESULT_{n}, RESULT
#
#	Note: The script should bear to be invoked multiple times
###############################################################

#Include common scripts
. /var/packages/MailPlus-Server/target/scripts/MailPlusServerCommon
. /var/packages/MailPlus-Server/target/backend_hook/hookUtils.conf

#At begining, acquire package settings
PackageInfoGet;

QUOTA_CLONE_UPDATE_SCRIPT="${PRIVATE_LOCATION}/scripts/QuotaCloneUpdate.sh"
REDIS_CLIENT="${PRIVATE_LOCATION}/usr/bin/redis-cli"
MAILBOX_TOOL="${PRIVATE_LOCATION}/bin/syno_mailbox"

case $1 in
	--sdk-mod-ver)
		#Print SDK support version
		echo "${SDK_MOD_VER}";
		;;
	--name)
		#Print package name
		echo "${MAIL_SERVER_PKG_NAME}";
		;;
	--pkg-ver)
		#Print package version
		echo "${MAIL_SERVER_PKG_VERSION}";
		;;
	--vendor)
		#Print package vendor
		echo "${MAIL_SERVER_PKG_VENDOR}";
		;;
	--pre)
		#noop
		;;
	--post)
		if [ "0" != "${RESULT}" ]; then
			exit 0
		fi

		usr_idx=1;
		usr_cnt=$(($NITEMS + 1));

		temp_user_list="$(mktemp)"
		while [ "$usr_idx" != "$usr_cnt" ]
		do
			eval "org_name=\$ORIGIN_USER_NAME_$usr_idx"
			eval "new_name=\$USER_NAME_$usr_idx"

			if [ ! -z "$org_name" ] && [ "$org_name" != "$new_name" ]; then
				$REDIS_CLIENT -p 8506 DEL "${org_name}/quota/messages" "${org_name}/quota/storage"
			fi
			echo "${new_name}" >> "${temp_user_list}"

			usr_idx=$(($usr_idx + 1));
		done
		$QUOTA_CLONE_UPDATE_SCRIPT update-from-file "${temp_user_list}"
		rm -r "${temp_user_list}"

		${MAILBOX_TOOL} quota-conf-generate
		;;
	*)
	echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
		;;
esac
