#!/bin/bash
. /var/packages/MailPlus-Server/target/backend_hook/hookUtils.conf
loadEnv

## Check identity
NODE_ID=$(getHostID)
storage_servers="$(${MAILPLUS_SERVER_BACKEND_BINARY} --getConfKeyVal "storager")"

LOCAL_DOMAIN_NAME="@local"

if ! containString "${NODE_ID}" "${storage_servers}"; then
	exit 0
fi

VOLUME_CONFIG_KEY="general_storage_volume-${NODE_ID}"
STORAGE_VOLUME="$(${MAILPLUS_SERVER_BACKEND_BINARY} --getConfKeyVal "${VOLUME_CONFIG_KEY}")"

if [ "${STORAGE_VOLUME}" == "" ]; then
	err_log "There are no storage volume"
	exit 0
fi

PACKAGE="MailPlus-Server"
PACKAGE_DIR="/var/packages/${PACKAGE}"
MAILDIR="Maildir"
ACL_AMEND_BIN="${PACKAGE_DIR}/target/bin/syno_acl_amend"

PrintTargetPath() {
	local mailAccountBaseDir
	if ! mailAccountBaseDir="$(GetMailAccountBaseDir)"; then
		err_log "It cannot get mailhome"
		return 1
	fi
	maildir_list=$(find "${mailAccountBaseDir}" -mindepth 3 -maxdepth 3 -type d -name "${MAILDIR}")
    (
        IFS=$'\n'
        for dir in ${maildir_list}
        do
            echo "${dir}"
        done
    )
	mailbox_list=$(find "${mailAccountBaseDir}" -mindepth 4 -maxdepth 4 -type d)
    (
        IFS=$'\n'
        for dir in ${mailbox_list}
        do
            echo "${dir}"
        done
    )
}

TransferACL() {
	local mailAccountBaseDir
	if ! mailAccountBaseDir="$(GetMailAccountBaseDir)"; then
		err_log "It cannot get mailhome"
		return 1
	fi
	maildir_list=$(find "${mailAccountBaseDir}" -mindepth 3 -maxdepth 3 -type d -name ${MAILDIR})
    (
        IFS=$'\n'
        for dir in ${maildir_list}
        do
             ${ACL_AMEND_BIN} -d "${dir}"
        done
    )
	mailbox_list=$(find "${mailAccountBaseDir}" -mindepth 4 -maxdepth 4 -type d)
    (
        IFS=$'\n'
        for dir in ${mailbox_list}
        do
             ${ACL_AMEND_BIN} -m "${dir}"
        done
    )
}

TransferACLForEsearch() {
	local mailAccountBaseDir
	if ! mailAccountBaseDir="$(GetMailAccountBaseDir)"; then
		err_log "It cannot get mailhome"
		return 1
	fi
	maildir_list=$(find "${mailAccountBaseDir}" -mindepth 3 -maxdepth 3 -type d -name ${MAILDIR})
    (
        IFS=$'\n'
        for dir in ${maildir_list}
        do
             ${ACL_AMEND_BIN} -e "${dir}"
        done
    )
}

while [ -n "$1" ]; do
case "$1" in
	"-d" | "--debug")
		PrintTargetPath
		exit 1
	;;
	"--esearch")
		TransferACLForEsearch
		exit 1
	;;
	*)
		echo "unhandled option '$1'"
		exit 1
	;;
esac
shift
done

if TransferACL; then
	exit 0
fi
exit 1
