#!/bin/bash

. /var/packages/MailPlus-Server/target/backend_hook/hookUtils.conf

PRIVATE_LOCATION="/var/packages/MailPlus-Server/target"

VOLUME=`/usr/bin/readlink ${PRIVATE_LOCATION} | /usr/bin/cut -d '/' -f2`

export PATH

run_sa_update()
{
	local anti_spam_update_status="/run/mailplus_server/antispam_update_status.json"
	local sa_update_log="${VOLUME}/@maillog/sa-update.log"
	local sa_update_folder="/var/spool/@MailPlus-Server/rspamd/sa_rules/update_db"
	local sa_update="${PRIVATE_LOCATION}/bin/sa-update"

	local status_tmp="${anti_spam_update_status}.tmp.$$"

	# set updating flag
	su MailPlus-Server -s /bin/sh -c "touch '${status_tmp}'"
	echo '{"status": "updating"}' > "${status_tmp}"
	mv "${status_tmp}" "${anti_spam_update_status}"

	# save last log to ${sa_update_log}.1
	[ -e "${sa_update_log}" ] && cp -a "${sa_update_log}" "${sa_update_log}.1"

	# run sa-update, and overwrite log file everytime
	su MailPlus-Server -s /bin/sh -c "${sa_update} --updatedir ${sa_update_folder} -D -v > ${sa_update_log} 2>&1"
	local code=$?

	# sa-update creates the empty dir, so delete it
	rmdir "${PRIVATE_LOCATION}/.spamassassin"

	# clear flag or set error flag
	if [ "0" -eq "${code}" ] || [ "1" -eq "${code}" ]; then
		rm "${anti_spam_update_status}"
	else
		su MailPlus-Server -s /bin/sh -c "touch '${status_tmp}'"
		echo "{\"status\": \"${code}\"}" > "${status_tmp}"
		mv "${status_tmp}" "${anti_spam_update_status}"

		# save log to bad log file if it failed for troubleshooting
		cp -a "${sa_update_log}" "${sa_update_log}.bad"
	fi
}

update_all()
{
	if isLoadBalancer; then
		nowTime=$(date "+%s")
		${MAILPLUS_SERVER_BACKEND_BINARY} --setConfKeyVal "spam_rule_last_update_time" "${nowTime}"
	fi
}

update_local()
{
	local rspamd_script="${PRIVATE_LOCATION}/scripts/daemon/rspamd.sh"
	local security_check="${PRIVATE_LOCATION}/scripts/SecurityCheck.sh"
	local sa_update_lock="/run/mailplus_server/update.lock"

	(
	if flock -x 8; then
		run_sa_update

		flock -u 8
		rm ${sa_update_lock}
	fi
	)8> ${sa_update_lock}

	${rspamd_script} status
	if [ 0 -eq $? ]; then
		${rspamd_script} restart
	else
		echo "Rspamd is not running."
	fi

	# update anti-spam status for dashboard
	${security_check} "anti_spam"
}


case "$1" in
	all)
		update_all
		;;
	local)
		update_local
		;;
	*)
		echo "Usage: $0 {all|local}" >&2
		exit 1
		;;
esac
