#!/bin/bash

. /var/packages/MailPlus-Server/target/scripts/daemon/util.sh

QUOTA_CLONE_PORT=8506
QUOTA_CLONE_DAEMON_PID="/var/run/mailplus_server/quota_clone.pid"
QUOTA_CLONE_PROC="redis-server 127.0.0.1:${QUOTA_CLONE_PORT}"
REDIS_SERVER="${MAIL_SERVER_PKG}/target/usr/bin/redis-server"
REDIS_CLI="${MAIL_SERVER_PKG}/target/usr/bin/redis-cli"
REDIS_CONF="${MAIL_SERVER_PKG}/target/etc/dovecot/dovecot-redis-quota-clone.conf.ext"
QUOTA_CLONE_UPDATE_SCRIPT="${MAIL_SERVER_PKG}/target/scripts/QuotaCloneUpdate.sh"

status()
{
	checkPidFileStatus "${QUOTA_CLONE_PROC}" "${QUOTA_CLONE_DAEMON_PID}"
	return $?
}

start()
{
	status
	if [ "$?" -eq "${SERVICE_RUNNING}" ]; then
		return ${START_SUCCESSFUL}
	fi
	echo "Starting Quota clone Daemon..."

	${REDIS_SERVER} ${REDIS_CONF}

	status
	if [ "$?" -eq "${SERVICE_RUNNING}" ] && [ -f "${QUOTA_CLONE_DAEMON_PID}" ]; then
		return ${START_SUCCESSFUL}
	else
		return ${START_FAILED}
	fi
}

checkStopped()
{
	waitTillPidRemoved "${QUOTA_CLONE_DAEMON_PID}"
	local status=$?
	if [ "${FAIL}" -eq "${status}" ]; then
		# Force stop
		killByPidFile "KILL" "${QUOTA_CLONE_DAEMON_PID}"
	fi
}

stop()
{
	status
	if [ "$?" -eq "${SERVICE_NOT_RUNNING}" ]; then
		return
	fi
    echo "Stop Quota clone Daemon..."
	${REDIS_CLI} -p ${QUOTA_CLONE_PORT} shutdown
	checkStopped
}

restart()
{
	stop
	sleep 1
	start
}

composePID()
{
	local pid=$(cat ${QUOTA_CLONE_DAEMON_PID} | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')
	echo "${pid}"
}

composeScript()
{
	echo "/var/packages/MailPlus-Server/target/scripts/daemon/quota_clone.sh"
}

case "$1" in
	start)
		startOperation "mailserver_service_quota_clone"
		;;
	restart)
		restartOperation "mailserver_service_quota_clone"
		;;
	stop)
		stopOperation "mailserver_service_quota_clone"
		;;
	status)
		status
		;;
	clear_and_update_in_background)
		${QUOTA_CLONE_UPDATE_SCRIPT} clear-and-update-all &
		;;
	*)
		echo "Usage: $0 {start|stop|restart|status|pid}" >&2
		exit 1
		;;
esac
